@@ -38,7 +38,7 @@
 #elif defined(DARWIN)
 # define MOD_NAME ".libs/mod_test.so" 
 # define LIB_NAME ".libs/libmod_test.dylib" 
-#elif defined(__hpux__) || defined(__hpux)
+#elif (defined(__hpux__) || defined(__hpux)) && !defined(__ia64)
 # define MOD_NAME ".libs/mod_test.sl"
 # define LIB_NAME ".libs/libmod_test.sl"
 #elif defined(_AIX) || defined(__bsdi__)
