@@ -1474,6 +1474,7 @@ static void test_datasync_on_stream(abts_case *tc, void *data)
     }
 }
 
+#if APR_HAS_THREADS
 typedef struct thread_file_append_ctx_t {
     apr_pool_t *pool;
     const char *fname;
@@ -1542,9 +1543,11 @@ static void * APR_THREAD_FUNC thread_file_append_func(apr_thread_t *thd, void *d
 
     return NULL;
 }
+#endif  /* APR_HAS_THREADS */
 
 static void test_atomic_append(abts_case *tc, void *data)
 {
+#if APR_HAS_THREADS
     apr_status_t rv;
     apr_status_t thread_rv;
     apr_file_t *f;
@@ -1597,6 +1600,9 @@ static void test_atomic_append(abts_case *tc, void *data)
     }
 
     apr_file_remove(fname, p);
+#else
+    ABTS_SKIP(tc, data, "This test requires APR thread support.");
+#endif /* APR_HAS_THREADS */
 }
 
 static void test_append_locked(abts_case *tc, void *data)
