@@ -309,7 +309,8 @@ static void overflow_strfsize(abts_case *tc, void *data)
     }
     for (off = LONG_MAX; off > 1; off /= 2) {
         apr_strfsize(off, buf);
-        apr_strfsize(off + 1, buf);
+        if (sizeof(apr_off_t) > sizeof(long) || off < LONG_MAX)
+            apr_strfsize(off + 1, buf);
         apr_strfsize(off - 1, buf);
     }
 
@@ -394,6 +395,19 @@ static void skip_prefix(abts_case *tc, void *data)
     ABTS_STR_EQUAL(tc, apr_cstr_skip_prefix("",      "12"),    NULL);
 }
 
+static void pstrcat(abts_case *tc, void *data)
+{
+    ABTS_STR_EQUAL(tc, apr_pstrcat(p, "a", "bc", "def", NULL),
+                   "abcdef");
+    ABTS_STR_EQUAL(tc, apr_pstrcat(p, NULL), "");
+    ABTS_STR_EQUAL(tc, apr_pstrcat(p,
+                                   "a", "b", "c", "d", "e",
+                                   "f", "g", "h", "i", "j",
+                                   "1", "2", "3", "4", "5",
+                                   NULL),
+                   "abcdefghij12345");
+}
+
 abts_suite *teststr(abts_suite *suite)
 {
     suite = ADD_SUITE(suite)
@@ -412,6 +426,7 @@ abts_suite *teststr(abts_suite *suite)
     abts_run_test(suite, string_cpystrn, NULL);
     abts_run_test(suite, snprintf_overflow, NULL);
     abts_run_test(suite, skip_prefix, NULL);
+    abts_run_test(suite, pstrcat, NULL);
 
     return suite;
 }
